#include "msg.h"

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>


extern char **environ;


bool set_env(const char *var)
	{
	return putenv((char *)var) == 0 || err(MSG_PUTENV, true, var);
	}

void unset_env(const char *var)
	{
	char **e;
	size_t len;
	bool inc = 0;

	len = strlen(var);
	for(e = environ; *e; ++e)
		{
		inc = inc || (strncmp(*e, var, len) == 0 && (*e)[len] == '=');
		e[0] = e[inc];
		}
	}

bool spawn(void (*run)(void *args), void *args, const char *m, const char *id)
	{
	int status;
	pid_t pid;

	pid = fork();
	if(pid < 0)
		die(MSG_FORK, true, NULL);
	if(pid == 0)
		run(args);
	waitpid(pid, &status, 0);
	return WEXITSTATUS(status) == 0 || err(m, false, id);
	}

void exec(void *args)
	{
	char **cmd = (char **)args;

	if(dup2(2, 1) != 1)
		die(MSG_STDIN_TO_STDERR, true, NULL);
	execvp(cmd[0], cmd);
	die(MSG_EXEC, true, cmd[0]);
	}
