#include "arch.h"
#include "extr.h"
#include "fs.h"
#include "main.h"
#include "mem.h"
#include "msg.h"
#include "str.h"

#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>


struct extractor *Extractors;


void match_extr(struct extractor *extr, const char *file)
	{
	regmatch_t m;

	extr->pos = (regexec(&extr->matcher, file, 1, &m, 0) == 0 &&
	             m.rm_so > 1 &&
	             file[m.rm_so - 1] == '.') ? m.rm_so : 0;
	}

struct extractor *find_extr(const struct archive *arch)
	{
	struct extractor *found = NULL, *extr;
	regoff_t leftmost = 0;

	for(extr = Extractors; extr; extr = extr->next)
		{
		match_extr(extr, arch->links.file.e);
		if(extr->pos > 0 &&
		   (!found || extr->pos < leftmost) &&
		   (!Extension || strcasecmp(Extension, arch->links.file.e + extr->pos) == 0))
			{
			found = extr;
			leftmost = extr->pos;
			if(Extension)
				break;
			}
		}
	return found;
	}

void add_extrs(const char *path)
	{
	DIR *dir;
	struct dirent *ent;
	struct extractor *extr;
	size_t path_len;
	char *abs, *file;

	dir = opendir(path);
	if(!dir)
		return;

	path_len = strlen(path);

	while((ent = readdir(dir)))
		{
		if(is_dot(ent->d_name))
			continue;

		extr = alloc(sizeof(struct extractor));

		abs = alloc(path_len + 1 + strlen(ent->d_name) + sizeof "/subdirectory");
		file = str_cp(str_cp(str_cp(abs, path), "("), ent->d_name);
		str_cp(file, ")$");
		if(regcomp(&extr->matcher, abs + path_len, REG_EXTENDED | REG_ICASE) != 0)
			{
			err(MSG_REGCOMP, true, abs + path_len);
			free(extr);
			free(abs);
			continue;
			}

		abs[path_len] = '/';
		str_cp(file, "/permissions");
		extr->perms = file_type(true, abs) > 1;
		str_cp(file, "/subdirectory");
		extr->subdir = file_type(true, abs) > 1;
		str_cp(file, "/extract");
		extr->extract = abs;

		extr->next = Extractors;
		Extractors = extr;
		}

	closedir(dir);
	}
