#include "exec.h"
#include "msg.h"
#include "path.h"

#include <dirent.h>
#include <errno.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>


char *ent_from_path(const char *path)
	{
	char *slash;

	slash = strrchr(path, '/');
	return slash ? slash + 1 : (char *)path;
	}

int file_type(bool follow, const char *path)
	{
	struct stat st;

	if((follow ? stat : lstat)(path, &st) == 0)
		return S_ISDIR(st.st_mode) ? 1 : 2;
	if(errno == ENOENT)
		return 0;
	err(MSG_STAT, true, path);
	return -1;
	}

bool get_inode(ino_t *inode, const char *path)
	{
	struct stat st;

	return lstat(path, &st) == 0 && (*inode = st.st_ino, true);
	}

bool is_dot(const char *file)
	{
	return file[0] == '.' && (!file[1] || (file[1] == '.' && !file[2]));
	}

DIR *opendir_verb(const char *name)
	{
	DIR *dir;

	dir = opendir(name);
	if(!dir)
		err(MSG_OPENDIR, true, name);
	return dir;
	}

DIR *open_path_dir(struct path p)
	{
	DIR *dir;
	char *sep;

	if(dlen(p) == 0)
		return opendir_verb(".");
	sep = p.e - 1;
	*sep = '\0';
	dir = opendir_verb(p.p);
	*sep = '/';
	return dir;
	}

void pretty_print(const char *path)
	{
	int type;

	/*
	 * file_type() may print an error message, so we call it right
	 * away to avoid interleaving output of stderr and stdout.
	 */
	type = file_type(false, path);
	WRITE(1, path);
	if(type == 1)
		WRITE(1, "/");
	WRITE(1, "\n");
	}

bool rm_rf(const char *path)
	{
	const char *args[] = { "rm", "-rf", "--", path, NULL };

	return spawn(exec, (void *)args, MSG_DEL, path);
	}

int safe_rename(const char *src, const char *dst)
	{
	/* caller ensures src != dst */

	bool dir = false;

	switch(file_type(false, src))
		{
		case -1: case 0: return -1;
		case 1: dir = true;
		}

	if((dir ? mkdir(dst, S_IRWXU | S_IRWXG | S_IRWXO): link(src, dst)) != 0)
		{
		if(errno == EEXIST)
			return 0;
		err(dir ? MSG_MKDIR : MSG_LINK, true, dst);
		return -1;
		}
	if((dir ? rename(src, dst) : unlink(src)) == 0)
		return 1;
	err(dir ? MSG_RENAME : MSG_DEL, true, src);
	if((dir ? rmdir(dst) : unlink(dst)) != 0)
		err(MSG_DEL, true, dst);
	return 0;
	}
